#!/bin/perl

# ============================================================================
# Copyright (c) 2011 by Terasic Technologies Inc. 
# ============================================================================
#
# Permission:
#
#   Terasic grants permission to use and modify this code for use
#   in synthesis for all Terasic Development Boards and Altera Development 
#   Kits made by Terasic.  Other use of this code, including the selling 
#   ,duplication, or modification of any portion is strictly prohibited.
#
# Disclaimer:
#
#   This VHDL/Verilog or C/C++ source code is intended as a design reference
#   which illustrates how these types of functions can be implemented.
#   It is the user's responsibility to verify their design for
#   consistency and functionality through the use of formal
#   verification methods.  Terasic provides no warranty regarding the use 
#   or functionality of this code.
#
# ============================================================================
#           
#                     Terasic Technologies Inc
#                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
#                     HsinChu County, Taiwan
#                     302
#
#                     web: http:#www.terasic.com/
#                     email: support@terasic.com
#
# ============================================================================
# Major Functions:	
#   TR4 Board Program flash tools
#
# ============================================================================
# Design Description:
#
# ===========================================================================
# Revision History :
# ============================================================================
#   Ver  :| Author            :| Mod. Date :| Changes Made:
#   V1.0 :| Eric Chen         :| 10/06/06  :| Initial Version
# ============================================================================




my $FLASH_0_BASE		= "0x08000000";
#my $FLASH_1_BASE		= "0x0A000000";
my $flash_bup_SOF		= "tr4_default_flash_loader.sof";
my $hw_image_file		= "tr4_hw.flash";
my $sw_image_file		= "tr4_sw.flash";
my $pfl_bits_file		= "tr4_hw.map.flash";
#my $zip_image_file		= "rozipfs.flash";


&init;
&menu;



sub	menu
{
	$SELECT = 100;
	until ($SELECT eq "D" or $SELECT eq "0")
	{
		system "clear";
		printf "TR4 Development Kit Flash Program Tools.  ver : 1.0.0.0\n";
		printf "************************   menu   ***********************************\n";
		printf "  0) Program .sof and .elf to flash (include pfl option bit)\n";
		printf "  1) Erase flash\n";
		printf "  2) Program .sof file into the flash\n";
		printf "  3) Program .elf file into the flash\n";
		printf "  4) Program pfl option bit into the flash\n";
		printf "  9) Readme File\n";
		printf "*********************************************************************\n";
		printf "Enter a number (D for Done): ";

		$SELECT = <STDIN>;   
		chop($SELECT);
		$SELECT =~ tr/a-z/A-Z/; 

		if	(					 ($SELECT eq "1"))	{&erase_flash;}			# erase flash
		if	(($SELECT eq "0") || ($SELECT eq "2"))	{&program_sof;}
		if	(($SELECT eq "0") || ($SELECT eq "3"))	{&program_elf;}
		if	(					 ($SELECT eq "4"))	{&program_pfl;}
		if  (					 ($SELECT eq "9"))	{
														system("cat readme.txt");
														printf "\nPress ENTER key to continuance... ";
														$RESULT = <STDIN>;
													} 
	} 
}


sub init
{
	
}

#==============================================================================
#==============================================================================
# +-------------------------
# | read_file(filename[,error_ref])
# |
# | returns the complete file contents
# |
# |
# |     in: filename -- name (path) of file to read
# |
# | return: one long string with the whole file, or
# |         empty string if no such file or empty file.
# |
# |         (use Perl's -e if you really really care whether
# |         the file was there...)
# |
sub read_file
{
  my ($filename,$error_out) = (@_);
  my $bunch;
  my $file_contents;
  my $did;

  # (return error if ref provided)

  if($error_out)
  {
    $$error_out = (-f $filename) ? 0 : -1;
  }

  # (read the whole file, if present. else "")

  if(open(FILE,$filename))
  {
    binmode FILE;    # Bite me, Windows! --dvb
    while(read(FILE,$bunch,32000))
    {
      $file_contents .= $bunch;
    }
    close FILE;
  }

  return $file_contents;
}
#==============================================================================
#==============================================================================
# -----------------------
# write_file(filename,contents)
#
# creates new file and writes entire
# file contents. Return "ok" if so,
# or "" if not.
#
# |
# |     in: filename -- name of file to create (or replace)
# |         contents -- string of all the bytes to put into file
# |
# | return: 0 for success, <0 for error
# |
sub write_file
{
  my ($filename,$contents) = (@_);

  my $did;

  #
  # If filename is "", print it to stdout
  # and that is all.
  #

  if($filename eq "")
  {
    print $contents;
    return "";
  }

  #
  # Delete existing file, if any.
  #

  unlink ($filename) if(-e $filename);

  $did = open(FILE,">$filename");
  if($did)
  {
    binmode FILE;    # Bite me, Windows! --dvb
    $did = print FILE $contents;
    close FILE;
    return $did ? 0 : -1;
  }

  return -1;
}
#==============================================================================
#==============================================================================
sub erase_flash
{
	# Load board update portal file into FPGA.
	printf "\nLoad board update portal file into FPGA, please wait ...\n\n";
	system "quartus_pgm -c USB-Blaster[USB-0] -m jtag -o p\\\;$flash_bup_SOF";

	# Programming flash with the FPGA configuration.
	printf "\nErase flash, please wait a few minutes ...\n\n";
	system "nios2-flash-programmer --base=$FLASH_0_BASE --cable='USB-Blaster [USB-0]' --erase-all";

	printf "\nPress ENTER key to continuance... ";
	$RESULT = <STDIN>;
}
#==============================================================================
#==============================================================================
sub program_sof
{
	my $SOF_FILE_OK = Y;
	my $SOF_FILE_NAME = "";
	system "clear";
	
	do
	{
		printf "Please input .sof file name : ";
		$SOF_FILE_OK = "Y";
		$SOF_FILE_NAME_IN = <STDIN>;
		chop ($SOF_FILE_NAME_IN);
		$SOF_FILE_NAME_IN =~ tr/a-z/A-Z/;
		$SOF_FILE_NAME_LEN = length($SOF_FILE_NAME_IN);

		if ($SOF_FILE_NAME_LEN == 0)
		{
			printf "Not enter any filename, please re-enter : ";
			$SOF_FILE_OK = "N";
		} 
		else
		{
			$POS = index($SOF_FILE_NAME_IN,".");	# Find symbol "." Location

			if ($POS == -1)							# if false(-1), insert ".sof"
			{
				$SOF_FILE_NAME = $SOF_FILE_NAME_IN . ".SOF";
			}
			elsif ($SOF_FILE_NAME_IN =~ /.SOF/)
			{
				$SOF_FILE_NAME = $SOF_FILE_NAME_IN;
			}
			else
			{
				printf "Input is not .sof the file name, please re-enter : ";
				$SOF_FILE_OK = "N";
			}
		}

		if (-e $SOF_FILE_NAME)
		{
			# Creating .flash file for the FPGA configuration.
			printf "\nStart file conversion, please wait a few minutes ...\n\n";
#			printf "$ENV{QUARTUS_ROOTDIR}\n";
			if ($ENV{QUARTUS_ROOTDIR} =~ /\/91\//)
			{
#				printf "for Quartus 11.0\n";
				system "sof2flash --offset=0x20000 --input=$SOF_FILE_NAME --output=$hw_image_file --pfl --programmingmode=FPP --optionbit=0x18000";
				&pfl_option_bit_proc;
			}
			else
			{
				printf "underfor Quartus 11.0\n";
				system "sof2flash --offset=0x20000 --input=$SOF_FILE_NAME --output=$hw_image_file --pfl --programmingmode=FPP --optionbit=0x18000";
				&pfl_option_bit_proc;
			}

			# Load board update portal file into FPGA.
			printf "\nLoad board update portal file into FPGA, please wait ...\n\n";
			system "quartus_pgm -c USB-Blaster[USB-0] -m jtag -o p\\\;$flash_bup_SOF";

			# Programming flash with the FPGA configuration.
			printf "\nProgram flash, please wait a few minutes ...\n\n";
			system "nios2-flash-programmer --base=$FLASH_0_BASE --cable='USB-Blaster [USB-0]' $hw_image_file";
			system "nios2-flash-programmer --base=$FLASH_0_BASE --cable='USB-Blaster [USB-0]' $pfl_bits_file";	
		}
		else
		{
			printf "\n";
			printf "Files do not exist,please make sure.\n";
			$SOF_FILE_OK = "N";
		}
	}
	while ($SOF_FILE_OK ne "Y");
	printf "\nPress ENTER key to continuance... ";
	$RESULT = <STDIN>;
}

#==============================================================================
#==============================================================================
sub program_elf
{
	my $ELF_FILE_OK = Y;
	my $ELF_FILE_NAME = "";
	system "clear";
	do
	{
		printf "Please input .ELF file name : ";
		$ELF_FILE_OK = "Y";
		$ELF_FILE_NAME_IN = <STDIN>;
		chop ($ELF_FILE_NAME_IN);
		$ELF_FILE_NAME_IN =~ tr/a-z/A-Z/;
		$ELF_FILE_NAME_LEN = length($ELF_FILE_NAME_IN);

		if ($ELF_FILE_NAME_LEN == 0)
		{
			printf "Not enter any filename, please re-enter : ";
			$ELF_FILE_OK = "N";
		} 
		else
		{
			$POS = index($ELF_FILE_NAME_IN,".");	# Find symbol "." Location

			if ($POS == -1)							# if false(-1), insert ".ELF"
			{
				$ELF_FILE_NAME = $ELF_FILE_NAME_IN . ".ELF";
			}
			elsif ($ELF_FILE_NAME_IN =~ /.ELF/)
			{
				$ELF_FILE_NAME = $ELF_FILE_NAME_IN;
			}
			else
			{
				printf "Input is not .elf the file name, please re-enter : ";
				$ELF_FILE_OK = "N";
			}
		}

		if (-e $ELF_FILE_NAME)
		{
			# Creating cfi_flash.flash file for the FPGA configuration.
			printf "\nStart file conversion, please wait a few minutes ...\n\n";
			system "elf2flash --base=0x08000000 --end=0xbffffff --reset=0xa020000 --input=$ELF_FILE_NAME --output=$tr4_sw.flash --boot=\"boot_loader_cfi.srec\"";

			# Load board update portal file into FPGA.
			printf "\nLoad board update portal file into FPGA, please wait ...\n\n";
			system "quartus_pgm -c USB-Blaster[USB-0] -m jtag -o p\\\;$flash_bup_SOF";

			# Programming flash with the FPGA configuration.
			printf "\nProgram flash, please wait a few minutes ...\n\n";
			system "nios2-flash-programmer --base=0x08000000 --cable='USB-Blaster [USB-0]' $tr4_sw.flash";
			
		}
		else
		{
			printf "\n";
			printf "Files do not exist,please make sure.\n";
			$ELF_FILE_OK = "N";
		}
	}
	while ($ELF_FILE_OK ne "Y");
	printf "\nPress ENTER key to continuance... ";
	$RESULT = <STDIN>;
}
#==============================================================================
#==============================================================================
sub program_pfl
{
	&pfl_option_bit_proc;
	printf "\nLoad board update portal file into FPGA, please wait ...\n\n";
	system "quartus_pgm -c USB-Blaster[USB-0] -m jtag -o p\\\;$flash_bup_SOF";
	printf "\nProgram flash, please wait a few minutes ...\n\n";
	system "nios2-flash-programmer --base=$FLASH_0_BASE --cable='USB-Blaster [USB-0]' $pfl_bits_file";	
}
#==============================================================================
sub pfl_option_bit_proc
{

	my $pfl_end_string = "S21501808003FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF6";

	if (-e $pfl_bits_file)
	{
		my $pfl_bit_string = read_file($pfl_bits_file);
		#  printf "pfl option bit file dump \n $pfl_bit_string\n\n"
		if ($pfl_bit_string =~ /$pfl_end_string/)
		{
			printf "File is ok, no change the file contents.\n";
		}
		else
		{
			write_file($pfl_bits_file,$pfl_bit_string . $pfl_end_string . "\n");
			printf "Modify tr4_hw.map.flash file ok.\n";
		}
	}
	else
	{
		printf "File does not exist, please be .sof file conversion.\n";
	}
}

#==============================================================================
#==============================================================================

